#!/usr/bin/env python3
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2024-04-24 13:08:21
FilePath     : /manifast/manifast/gui_utils/summarize.py
Description  : 
LastEditTime : 2024-04-24 16:22:56
LastEditors  : knightdby
Copyright (c) 2024 by Inc, All Rights Reserved.
'''
import itertools
from tabulate import tabulate

def precision_summarize(precisions_dict,headers=["category", "AP"]):

    """
    Summarize the precision results in a table.

    Args:
        precisions_dict (dict): a dictionary containing the precision results.

    Returns:
        str: the table as a string.
    """
    sorted_dict = {k: precisions_dict[k] for k in sorted(precisions_dict)}
    results_per_category=sorted_dict.items()
    N_COLS = min(6, len(results_per_category) * 2)
    results_flatten = list(
        itertools.chain(*results_per_category))
    results_2d = itertools.zip_longest(
        *[results_flatten[i::N_COLS] for i in range(N_COLS)])
    table = tabulate(
        results_2d,
        tablefmt="pipe",
        floatfmt=".3f",
        headers=headers * (N_COLS // 2),
        numalign="left",
    )
    return table

def precision_summarize_small(small_dict):
    """
    Create a small table using the keys of small_dict as headers. This is only
    suitable for small dictionaries.

    Args:
        small_dict (dict): a result dictionary of only a few items.

    Returns:
        str: the table as a string.
    """
    sorted_dict = {k: small_dict[k] for k in sorted(small_dict)}
    keys, values = tuple(zip(*sorted_dict.items()))
    table = tabulate(
        [values],
        headers=keys,
        tablefmt="pipe",
        floatfmt=".3f",
        stralign="center",
        numalign="center",
    )
    return table


def experiment_summarize(experiment_dict_):
    """
    Create a summary of an experiment.

    Args:
        experiment_dict_ (dict): a dictionary containing the results of an
            experiment.
             dictionary has the following format:
            {'name': {'score': xxx, 'precision_div': {}, 'precision_sum': {}}}

    Returns:
        str: the summary as a string.
    """
    experiment_dict = {key: value for key, value in sorted(
    experiment_dict_.items(), key=lambda item: float(item[1]['score']), reverse=True)}

    summarize_text=''
    # summarize_text+="-----------------summary-----------------\n"
    score_sum=[['exp_name','score']]
    for key, sub_dict in experiment_dict.items():
        score_sum.append([key,sub_dict['score']]) 
    score_info=tabulate(score_sum,
                       tablefmt= "heavy_outline",
                        headers="firstrow",
                             showindex="always",
                        numalign="center",
                             )+'\n\n'
    print(score_info)
    summarize_text+=score_info
    # summarize_text+="\n-----------------dividual-----------------\n"
    for key, sub_dict in experiment_dict.items():
        summarize_text+=f"name:  {key}\n"
        summarize_text+=f"score: {sub_dict['score']}\n"
        summarize_text+='++++'*20+'\n'
        
        for key_i, value_i in sub_dict.items():
            if '_sum' in key_i:
                key_i=key_i.replace('_sum','')
                # summarize_text+=f"{key_i}:\n"
                table = precision_summarize_small(value_i)+'\n'
                summarize_text+=table
        summarize_text+='++++'*20+'\n'
        for key_i, value_i in sub_dict.items():
            if '_div' in key_i:
                key_i=key_i.replace('_div','')
                # summarize_text+=f"{key_i}:\n"
                table = precision_summarize(value_i)+'\n'
                summarize_text+=table
        summarize_text+='++++'*20+'\n'
        for key_i, value_i in sub_dict.items():
            if '_sum' not in key_i and '_div' not in key_i and 'score' not in key_i:
                # summarize_text+=f"{key_i}:\n"
                summarize_text+=f"{sub_dict}:\n"
        summarize_text+='++++'*20+'\n\n\n'
    return summarize_text



#!/usr/bin/env python3
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2024-04-23 14:10:33
FilePath     : /manifast/manifast/label_utils/segment/get_mask_shapes.py
Description  : 
LastEditTime : 2024-04-23 14:39:16
LastEditors  : knightdby
Copyright (c) 2024 by Inc, All Rights Reserved.
'''
import cv2
import numpy as np
currentCV_version = cv2.__version__  # str

def getBinary(img_or_path, minConnectedArea=20):
    if isinstance(img_or_path, str):
        i = cv2.imread(img_or_path)
    elif isinstance(img_or_path, np.ndarray):
        i = img_or_path
    else:
        raise TypeError('Input type error')

    if len(i.shape) == 3:
        img_gray = cv2.cvtColor(i, cv2.COLOR_BGR2GRAY)

    else:
        img_gray = i

    ret, img_bin = cv2.threshold(img_gray, 127, 255, cv2.THRESH_BINARY)

    _, labels, stats, centroids = cv2.connectedComponentsWithStats(img_bin)
    # print(stats.shape)
    for index in range(1, stats.shape[0]):
        if stats[index][4] < minConnectedArea or stats[index][4] < 0.1 * (
                stats[index][2] * stats[index][3]):
            labels[labels == index] = 0

    labels[labels != 0] = 1

    img_bin = np.array(img_bin * labels).astype(np.uint8)
    # print(img_bin.shape)

    return i, img_bin


def get_approx(img, contour, length_p=0.1):
    """获取逼近多边形

    :param img: 处理图片
    :param contour: 连通域
    :param length_p: 逼近长度百分比
    """
    img_adp = img.copy()
    # 逼近长度计算
    epsilon = length_p * cv2.arcLength(contour, True)
    # 获取逼近多边形
    approx = cv2.approxPolyDP(contour, epsilon, True)

    return approx


def getMultiRegion(img, img_bin):
    """
    for multiple objs in same class
    """
    # tmp = currentCV_version.split('.')
    if float(currentCV_version[0:3]) < 3.5:
        img_bin, contours, hierarchy = cv2.findContours(
            img_bin, cv2.RETR_LIST, cv2.CHAIN_APPROX_SIMPLE)
    else:
        contours, hierarchy = cv2.findContours(img_bin, cv2.RETR_LIST,
                                               cv2.CHAIN_APPROX_SIMPLE)
    # print(contours)
    # print(len(contours))
    regions = []
    if len(contours) >= 1:

        for i in range(0, len(contours)):
            area = cv2.contourArea(contours[i])
            region = get_approx(img, contours[i], 0.002)
            if region.shape[0] > 3 and area > 20:
                regions.append(region)
        return regions
    else:
        return []
    

def getPolygonwithMask(oriImg):
    img, img_bin = getBinary(oriImg)

    return getMultiRegion(img, img_bin)
